@@ -113,7 +113,7 @@ def isointerp2(iso1,iso2,frac,photnames=None,minlabel=1,maxlabel=7,verbose=False
         colnames = np.char.array(iso1.colnames)
         photind, = np.where((colnames.find('MAG')>-1) & (colnames.find('_')>-1))
         photnames = list(colnames[photind])
-    interpnames = ['INT_IMF','MASS']+photnames
+    interpnames = ['INT_IMF','MASS','LOGTE','LOGG']+photnames
 
     # Initialize the output catalog
     nout = int(1.5*np.max([niso1,niso2]))
@@ -122,7 +122,9 @@ def isointerp2(iso1,iso2,frac,photnames=None,minlabel=1,maxlabel=7,verbose=False
     out['METAL'] = metal1*(1-frac)+metal2*frac
     out['MINI'] = 0.0
     out['INT_IMF'] = 0.0
-    out['MASS'] = 0.0        
+    out['MASS'] = 0.0
+    out['LOGTE'] = 0.0
+    out['LOGG'] = 0.0            
     out['LABEL'] = 0
     for n in photnames:
         out[n] = 0.0
@@ -212,11 +214,18 @@ def isointerp2(iso1,iso2,frac,photnames=None,minlabel=1,maxlabel=7,verbose=False
 
         # Single data point, often for label=9, WD
         else:
-            out['MINI'][count] = iso1['MINI'][lab1]*(1-frac)+iso2['MINI'][lab2]*frac
+            uselab1, = np.where(lab1)
+            uselab2, = np.where(lab2)
+            # take middle point
+            if len(uselab1)==1 and len(uselab2)>1:
+                uselab2 = uselab2[len(uselab2)//2]
+            elif len(uselab2)==1 and len(uselab1)>1:
+                uselab1 = uselab1[len(uselab1)//2]
+            out['MINI'][count] = iso1['MINI'][uselab1]*(1-frac)+iso2['MINI'][uselab2]*frac
             for n in interpnames:
-                out[n][count] = iso1[n][lab1]*(1-frac)+iso2[n][lab2]*frac
+                out[n][count] = iso1[n][uselab1]*(1-frac)+iso2[n][uselab2]*frac
             out['LABEL'][count] = l
-            count += nlab1
+            count += 1
 
         
         #import pdb; pdb.set_trace()
@@ -355,8 +364,9 @@ class Isochrone:
         if extdict is not None:
             self._extdict = extdict
         else:
-            self._extdict = extinction.load()
-        
+            extdict,exttab = extinction.load()
+            self._extdict = extdict
+            self._exttab = exttab
         
     def __call__(self,distmod=None,ext=None,maxlabel=None):
         """ Return an isochrone with a given distance modulus and extinction."""
@@ -470,23 +480,16 @@ class Isochrone:
         """ Return the list of bands."""
         return self._bands
 
-    def synth(self,nstars=1000,minlabel=1,maxlabel=8,names=None,minmass=0,maxmass=1000):
+    def synth(self,nstars=None,totmass=None,minlabel=1,maxlabel=8,names=None,minmass=0,maxmass=1000):
         """ Create synthetic population."""
     
         if names is None:
             names = self.bands
-        
-        # Initialize the output catalog
-        out = Table()
-        out['AGE'] = np.zeros(int(nstars),float)
-        out['METAL'] = 0.0
-        out['MINI'] = 0.0
-        #out['INT_IMF'] = 0.0
-        out['MASS'] = 0.0        
-        out['LABEL'] = 0
-        for n in names:
-            out[n] = 0.0        
-        
+
+        # By default us 1000 stars
+        if nstars is None and totmass is None:
+            nstars = 1000
+
         lab = ((self._data['LABEL']>=minlabel) & (self._data['LABEL']<=maxlabel))
         data = self._data[lab]
 
@@ -496,7 +499,25 @@ class Isochrone:
         ndata = len(data)
         if ndata==0:
             raise ValueError('No isochrone points left after mass cut')
-        
+
+        # Total mass input, figure out the number of stars we expect
+        # for our stellar mass range
+        if nstars is None and totmass is not None:
+            nstars = np.ceil((np.max(data['INT_IMF'])-np.min(data['INT_IMF']))*totmass)
+            
+        # Initialize the output catalog
+        out = Table()
+        out['AGE'] = np.zeros(int(nstars),float)
+        out['METAL'] = 0.0
+        out['MINI'] = 0.0
+        #out['INT_IMF'] = 0.0
+        out['MASS'] = 0.0
+        out['LOGTE'] = 0.0
+        out['LOGG'] = 0.0                
+        out['LABEL'] = 0
+        for n in names:
+            out[n] = 0.0        
+
         # PDF, probability distribution function
         # int_IMF, which is the integral of the IMF under consideration (as selected in the form, in number of stars,
         # and normalised to a total mass of 1 Msun) from 0 up to the current M_ini. Differences between 2 values of
@@ -594,9 +615,14 @@ class IsoGrid:
         npoints = []
         ind2ind1 = np.zeros((len(uages),len(umetals)),int)
         count = 0
+        ageindex = dln.create_index(iso['AGE'])
         for i,a in enumerate(uages):
-            for j,m in enumerate(umetals):
-                ind, = np.where((iso['AGE']==a) & (iso['METAL']==m))
+            aind = ageindex['index'][ageindex['lo'][i]:ageindex['hi'][i]+1]
+            mindex = dln.create_index(iso[aind]['METAL'])
+            for j,m in enumerate(mindex['value']):
+                mind = mindex['index'][mindex['lo'][j]:mindex['hi'][j]+1]
+                ind = aind[mind]
+                #ind, = np.where((iso['AGE']==a) & (iso['METAL']==m))
                 index.append(ind)
                 allages.append(a)
                 allmetals.append(m)
@@ -613,8 +639,10 @@ class IsoGrid:
         if extdict is not None:
             self._extdict = extdict
         else:
-            self._extdict = extinction.load()
-
+            extdict,exttab = extinction.load()
+            self._extdict = extdict
+            self._exttab = exttab
+            
         
     def __repr__(self):
         """ Print out string representation."""
@@ -675,7 +703,7 @@ class IsoGrid:
             names = list(np.array(self.bands)[nameind])
         if names is None:
             names = self.bands
-        outnames = ['AGE','METAL','MINI','INT_IMF','MASS','LABEL']+names
+        outnames = ['AGE','METAL','MINI','INT_IMF','MASS','LOGTE','LOGG','LABEL']+names
         
         
         # Get the closest isochrone on the grid
